/*
 *  GSet
 *  Copyright (C) 2001  William H Darrow 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/*  Quick multiplayer documentation 
 *  Client -> Server relationship
 *  	'A' signals authentication message following
 *  	'B' signals issue the begin command, you must be authenticated
 *  	'G' signals tell the server you have found a set
 *  	'X' signals tell the server you chose a bad set
 *  	'U' signals tell the server you are sending a username
 *  Server -> Client relationship
 *  	'F' signals tell that an initial field will be delivered in the following send()
 *  	'S' signals tell that an array containing the field sets will be delivered in the following send()
 *  	'M' signals tell that a message from the server should be displayed in the statusbar
 *  	'N' signals tell that a new field is going to be sent to the client and the client's score should not be reset
 *  	'I' signals tell that the score sheet is about to be sent
 *  	'C' signals tell the client which cards to remove from the table
 *  	'Q' signals show the reason why the server is disconnecting the client
 *  	'Z' signals tell the client to send a ping response
 */ 
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <gtk/gtk.h>
#include <config.h>

struct userif {
	int filedesc;
	int score;
	int auth;
	char username[8];
};

void start_recv(char *entry_text, int k, char *entry_textc);
void send_auth(char *entry_text);
void trigger();
void mnew_field();
void clrall();
void parse_sets();
void mnew_game();
void send_sets();
void parse_scores();
void fix_table(int *xtr);
void lower_score();

int field[12][4];
int set[12];
int sockfd;
GtkWidget *pixmap[15];
GtkWidget *status_bar;
GtkWidget *window;
GtkWidget *label;
int number_sets;
int sets_in_field;
char blank[100];
int identifier;
int fieldsets[9][3];
int connected;
int from_a;
int from_b;
int from_c;
int moverride;
char statustext[100];
char name[8];
struct userif users[CLIMIT];

void start_recv(char *entry_text, int k, char *entry_textc) { 
	struct hostent *he;
	struct sockaddr_in host_addr; 
	clrall();
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	host_addr.sin_family = AF_INET; 
	host_addr.sin_port = htons(k);
	if ((he=gethostbyname(entry_text)) == NULL) {
		strcpy(statustext, "Could not resolve host");
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
		return;
	}
	host_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(host_addr.sin_zero), '\0', 8);
	if(connect(sockfd, (struct sockaddr *)&host_addr, sizeof(struct sockaddr)) == -1) {
		strcpy(statustext, "Connection Refused");
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
	} else {
		strcpy(name, "U");
		strcat(name, entry_textc);
		strcpy(statustext, "Connected to host; Ready to recieve field!");
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
		identifier = gdk_input_add(sockfd, GDK_INPUT_READ, trigger, NULL);
		send(sockfd, name, strlen(name)+1, 0);
		connected = 1;
	}
}

void send_auth(char *entry_text) {
	char passtest[26];
	if(strlen(entry_text) <=25) {
		strcpy(passtest, "A");
		strcat(passtest, entry_text);
		send(sockfd, passtest, strlen(passtest)+1, 0);
	} else {
		strcpy(statustext, "Password exceeds 25 characters so not sent");
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
	}
}

void trigger() {
	int numbytes;
	int x,k;
	char buffer[1];
	int playernum[1];
	int xtr[3];
	if ((recv(sockfd, buffer, sizeof(buffer), 0)) <= 0) {
		if (moverride != 1) {
			strcpy(statustext, "Remote host closed the connection; Disconnected");
		}
		moverride = 0;
		connected = 0;
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
		gdk_input_remove(identifier);
		close(sockfd);
		clrall();
	} else {
		switch(buffer[0]) {
			case 'F': 
				recv(sockfd, field, sizeof(int)*12*4, 0); 
				mnew_game(); 
				break;
			case 'S': 
				recv(sockfd, fieldsets, sizeof(int)*9*3, 0); 
				parse_sets();
				break;
			case 'M':
				recv(sockfd, statustext, sizeof(statustext), 0); 
				gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
				break;
			case 'N':
				recv(sockfd, field, sizeof(int)*12*4, 0); 
				mnew_field();
				break;
			case 'I':
				recv(sockfd, users, (sizeof(int)*3*CLIMIT)+(sizeof(users[0].username)*CLIMIT), 0);
				parse_scores();
				break;
			case 'C':
				recv(sockfd, xtr, sizeof(xtr), 0);
				fix_table(xtr);
				break;
			case 'Q':
				moverride = 1;
				recv(sockfd, statustext, sizeof(statustext), 0);
				break;
			case 'Z':
				send(sockfd, "Z", 1, 0);
				break;
		}
	}
}

void mnew_game() {
	number_sets = 0;
	mnew_field();
}

void mnew_field() {
	GdkPixmap *image;
	GdkPixmap *mask;
	char filename[100];
	char statustext[100];
	char temp[10];
	int i, x;
	if(connected != 1) {
		sprintf(statustext, "Number of sets in field: %d   Your score: %d", sets_in_field, number_sets);
		gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
	}
	for(x=0; x<=11; x++) {
		strcpy(filename, PIXPATH);
		for(i=0; i<=3; i++) {
			sprintf(temp, "%i", field[x][i]);
			strcat(filename, temp);
		}
		strcat(filename, ".xpm");
		image = gdk_pixmap_create_from_xpm(window->window, NULL, NULL, filename);
		gtk_pixmap_set(GTK_PIXMAP(pixmap[x]), image, NULL);
	}
	for(i=0; i<=11; i++) {
		set[i] = 0;
	}
	image = gdk_pixmap_create_from_xpm(window->window, &mask, NULL, blank);
	gtk_pixmap_set(GTK_PIXMAP(pixmap[12]), image, mask);
	gtk_pixmap_set(GTK_PIXMAP(pixmap[13]), image, mask);
	gtk_pixmap_set(GTK_PIXMAP(pixmap[14]), image, mask);
}
	
void clrall() {
	GdkPixmap *image;
	GdkPixmap *mask;
	int i,x,k;
	for(i=0; i<=11; i++) {
		set[i] = 0;
	}
	for(x=0; x<=11; x++) {
		for(i=0; i<=3; i++) {
			field[x][i] = 0;
		}
	}
	for(i=0;i<=8;i++) {
		for(x=0;x<=2;x++) {
			fieldsets[i][x] = 0;
		}
	}
	image = gdk_pixmap_create_from_xpm(window->window, &mask, NULL, blank);
	for(i=0;i<=14;i++) {
		gtk_pixmap_set(GTK_PIXMAP(pixmap[i]), image, mask);
	}
}
	
void parse_sets() {
	int i,k;
	char statustext[100];
	sets_in_field = 0;
	for(i=0;i<=8;i++) {
		if(fieldsets[i][1] != 0) {
			sets_in_field++;
		}
	}
	sprintf(statustext, "Number of sets in field: %d   Your score: %d", sets_in_field, number_sets);
	gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1, statustext);
}

void send_sets() {
	char xtr[5];
	xtr[0] = 'G';
	xtr[1] = from_a;
	xtr[2] = from_b;
	xtr[3] = from_c;
	send(sockfd, xtr, sizeof(xtr), 0);
}

void parse_scores() {
	int x;
	char labeltext[300];
	char eachtext[100];
	strcpy(labeltext, "\0");
	for(x=0;x<CLIMIT;x++) {
		if(users[x].score >= 0) {
			sprintf(eachtext, "%s:  %d\n", users[x].username, users[x].score);
			strcat(labeltext, eachtext);
		}
	}
	gtk_label_set_text(GTK_LABEL(label), labeltext);
}

void fix_table(int *xtr) {
	GdkPixmap *image;
	GdkPixmap *mask;
	int i, k;
	image = gdk_pixmap_create_from_xpm(window->window, &mask, NULL, blank);
	for(i=0; i<=2; i++) {
		for(k=0; k<=3; k++) {
			field[xtr[i]][k] = 0;
			gtk_pixmap_set(GTK_PIXMAP(pixmap[xtr[i]]), image, mask);
		}
	}
	if (set[0] != 0) {
		if((from_a == xtr[0]) || (from_a == xtr[1]) || (from_a == xtr[2])) {
			gtk_pixmap_set(GTK_PIXMAP(pixmap[12]), image, mask);
			set[0] = 0; set[1] = 0; set[2] = 0; set[3] = 0;
		}
	}
	if (set[4] != 0) {
		if((from_b == xtr[0]) || (from_b == xtr[1]) || (from_b == xtr[2])) {
			gtk_pixmap_set(GTK_PIXMAP(pixmap[13]), image, mask);
			set[4] = 0; set[5] = 0; set[6] = 0; set[7] = 0;
		}
	}
	if (set[8] != 0) {
		if((from_c == xtr[0]) || (from_c == xtr[1]) || (from_c == xtr[2])) {
			gtk_pixmap_set(GTK_PIXMAP(pixmap[14]), image, mask);
			set[8] = 0; set[9] = 0; set[10] = 0; set[11] = 0;
		}
	}		 
}
	
void lower_score() {
	send(sockfd, "X", 1, 0);
}
	
	
	
	
	
	
	
	